<?php

/**
 * @file
 * Helper functions related to taxonomy.
 */


/**
 * Get term id for a given vocabulary, node field name  and node field value.
 *
 * @param string $vid
 *   Vocabulary identifier.
 *
 * @param string $field
 *   The node field which is tied to the vocabulary.
 *
 * @param string $value
 *   Value to compare with vocabulary terms.
 *
 * @return int
 *   Term identifier or NULL.
 */
function _opac_taxonomy_get_term($vid, $field, $value) {
  $query = new EntityFieldQuery();

  if ($field == 'name') {
    $query->entityCondition('entity_type', 'taxonomy_term', '=')
      ->propertyCondition('name', $value, '=')
      ->propertyCondition('vid', $vid, '=');
  }
  else {
    $query->entityCondition('entity_type', 'taxonomy_term', '=')
      ->propertyCondition('vid', $vid, '=')
      ->fieldCondition($field, 'value', $value, '=');
  }

  $terms = $query->execute();
  if (!isset($terms['taxonomy_term'])) {
    return 0;
  }
  $tid = key($terms['taxonomy_term']);
  return $tid;
}

/**
 * Create a new term for a given vocabulary.
 *
 * @param string $name
 *   The value of term name field to create.
 *
 * @param int $vid
 *   The vocabulary identifier for which create teh term.
 *
 * @return int
 *   Term identifier newly created.
 */
function _opac_taxonomy_new_term($name, $vid) {
  $term = (object) array(
    'name' => $name,
    'vid' => $vid,
  );
  $status = taxonomy_term_save($term);
  return $term->tid;
}
