<?php

/**
 * @file
 * Taxonomy plugin.
 *
 * This plugin automatically create link to taxonomy terms.
 * It also allows to create taxonomy terms on the fly.
 *
 * It only works with taxonomy_term_reference fields.
 */


/**
 * Callback 'available' for taxonomy plugin.
 */
function opac_plugin_taxonomy_available($server, $field) {
  if ($field['type'] == 'taxonomy_term_reference') {
    return TRUE;
  }
  return FALSE;
}

/**
 * Callback for taxonomy plugin.
 */
function opac_plugin_taxonomy($record, $mapping) {
  module_load_include('inc', 'opac', 'includes/helper.taxonomy');
  $remotefield = $mapping['mapped_with'];
  $field = array('und' => array());
  if (isset($record[$remotefield])) {
    foreach ($record[$remotefield] as $value) {
      // Retieves the term id depending of the field value.
      $tid = _opac_taxonomy_get_term($mapping['vid'], $mapping['term_field'], $value);
      if ($tid) {
        $field['und'][] = array('tid' => $tid);
      }
      // No term match the field value. So if nomatch_rule
      // is 'create', we add a new term: The remotefield value.
      elseif ($mapping['nomatch_rule'] == 'create') {
        $tid = _opac_taxonomy_new_term($value, $mapping['vid']);
        $field['und'][] = array('tid' => $tid);
      }

      // If we did not find a taxo id, and we have a default value,
      // we use it.
      else {

        // Getting fields info, in order to check for a default value.
        $field_infos = field_info_instance('node', $nodefield, $node->type);

        // If we have a default value, we use it.
        $default_tid = $field_infos['default_value'][0]['tid'];
        if ($default_tid) {
          $field['und'][] = array('tid' => $default_tid);
        }
      }
    }
  }
  return $field;
}

/**
 * Additional form elements for taxonomy.
 */
function opac_plugin_taxonomy_form(&$form_state, $field_settings, $mapping) {
  $new = count($mapping['mapped']) ? 0 : 1;
  $vocabulary_machine_name = $field_settings['settings']['allowed_values'][0]['vocabulary'];
  $vocabulary = taxonomy_vocabulary_machine_name_load($vocabulary_machine_name);
  $vocabulary_name = $vocabulary->name;

  $vocabulary_fields = field_info_instances('taxonomy_term', $vocabulary_machine_name);
  $vocabulary_fields_opt = array('name' => 'Term name');
  foreach ($vocabulary_fields as $name => $field) {
    $vocabulary_fields_opt[$name] = $field['label'];
  }

  $form['vocabulary_machine_name'] = array(
    '#type' => 'hidden',
    '#value' => $vocabulary_machine_name,
  );
  $form['vid'] = array(
    '#type' => 'hidden',
    '#value' => $vocabulary->vid,
  );
  $form['vocabulary_name'] = array(
    '#type' => 'hidden',
    '#value' => $vocabulary_name,
  );
  $form['taxonomy_term_field_label_div'] = array(
    '#prefix' => '<div id="taxonomy_term_field_label_div">',
    '#suffix' => '</div>',
  );
  $values = isset($form_state['values']) ? $form_state['values'] : NULL;

  $term_field;
  if ($values) {
    $term_field = $values['term_field'];
  }
  else {
    $term_field = $new ? '' : $mapping['mapped'][0]['term_field'];
  }

  $term_field_label;
  if ($term_field && $term_field == 'name') {
    $term_field_label = t('Term name');
  }
  elseif ($term_field) {
    $term_field_label = $vocabulary_fields[$term_field]['label'];
  }

  $form['taxonomy_term_field_label_div']['term_field_label'] = array(
    '#type' => 'hidden',
    '#value' => $term_field_label,
  );
  $form['taxonomy_rule'] = array(
    '#type' => 'fieldset',
    '#title' => check_plain(t("Taxonomy rule for vocabulary $vocabulary_name")),
    '#attributes' => array('class' => array('container-inline')),
  );
  $form['taxonomy_rule']['term_field'] = array(
    '#type' => 'select',
    '#title' => t('Term field to compare'),
    '#required' => TRUE,
    '#default_value' => $new ? NULL : $mapping['mapped'][0]['term_field'],
    '#options' => $vocabulary_fields_opt,
    '#ajax' => array(
      'callback' => 'opac_plugin_taxonomy_term_field_label_callback',
      'wrapper' => 'taxonomy_term_field_label_div',
    ),
  );
  $form['taxonomy_rule']['nomatch_rule'] = array(
    '#title' => 'No match rule',
    '#type' => 'select',
    '#options' => array('none' => 'Leave empty', 'create' => 'Create new term'),
    '#default_value' => $new ? 'none' : $mapping['mapped'][0]['nomatch_rule'],
  );
  return $form;
}

/**
 * Callback for opac_plugin_taxonomy_form.
 */
function opac_plugin_taxonomy_term_field_label_callback($form, &$form_state) {
  return $form['plugin']['options']['taxonomy_term_field_label_div'];
}
